/* base styles, using foundation 6 defaults */
html {
  font-family:sans-serif;
  line-height:1.15;
  -ms-text-size-adjust:100%;
  -webkit-text-size-adjust:100%;
}

article,aside,footer,header,nav,section {
  display:block;
}

a {
  background-color:transparent;
  -webkit-text-decoration-skip:objects;
}
a:active,a:hover {
  outline-width:0;
}

img {
  border-style:none;
}

html {
  box-sizing:border-box;
  font-size:100%;
}
*,*::before,*::after {
  box-sizing:inherit;
}
body /* base font and color for privacy badger */ {
  margin:0;
  padding:0;
  background:#fefefe;
  font-family:"OpenSans-Light",sans-serif;
  font-weight:normal;
  line-height:1.5;
  color:#000;
  -webkit-font-smoothing:antialiased;
  -moz-osx-font-smoothing:grayscale;
  overflow-x: hidden;
}
img {
  display:inline-block;
  vertical-align:middle;
  max-width:100%;
  height:auto;
  -ms-interpolation-mode:bicubic;
}

/* xy grid, selected foundation classes */
.small-12 {
  flex:0 0 100%;
  max-width:100%;
}
.grid-container {
  padding-right:.625rem;
  padding-left:.625rem;
  max-width:75rem;
  margin:0 auto;
}
@media print, screen and (min-width: 40em) {
  .grid-container {
    padding-right:.9375rem;
    padding-left:.9375rem;
  }
}

.grid-container.full {
  padding-right:0;
  padding-left:0;
  max-width:100%;
  margin:0 auto;
}
.grid-x {
  display:flex;
  flex-flow:row wrap;
}
.cell {
  flex:0 0 auto;
  min-height:0px;
  min-width:0px;
  width:100%;
}

.grid-x>.small-shrink,.grid-x>.small-full,.grid-x>.small-1,.grid-x>.small-2,.grid-x>.small-3,.grid-x>.small-4,.grid-x>.small-5,.grid-x>.small-6,.grid-x>.small-7,.grid-x>.small-8,.grid-x>.small-9,.grid-x>.small-10,.grid-x>.small-11,.grid-x>.small-12 {
  flex-basis:auto;
}
@media print, screen and (min-width: 40em) {
  .grid-x>.medium-10 {
    flex-basis:auto;
  }

}
@media print, screen and (min-width: 64em) {
  .grid-x>.large-3,.grid-x>.large-4 {
    flex-basis:auto;
  }
}
.grid-x>.small-12 {
  width:100%;
}
@media print, screen and (min-width: 40em) {
  .grid-x>.medium-10 {
    width:83.33333%;
  }
}
@media print, screen and (min-width: 64em) {
  .grid-x>.large-3 {
    width:25%;
  }
  .grid-x>.large-4 {
    width:33.33333%;
  }
}
.grid-margin-x {
  margin-left:-.625rem;
  margin-right:-.625rem;
}
@media print, screen and (min-width: 40em) {
  .grid-margin-x {
    margin-left:-.9375rem;
    margin-right:-.9375rem;
  }
}

.grid-margin-x>.small-12 {
  width:calc(100% - 1.25rem);
}
@media print, screen and (min-width: 40em) {

  .grid-margin-x>.small-12 {
    width:calc(100% - 1.875rem);
  }
  .grid-margin-x>.medium-10 {
    width:calc(83.33333% - 1.875rem);
  }
}
@media print, screen and (min-width: 64em) {
  .grid-margin-x>.large-3 {
    width:calc(25% - 1.875rem);
  }
  .grid-margin-x>.large-4 {
    width:calc(33.33333% - 1.875rem);
  }
}
.grid-padding-x .grid-padding-x {
  margin-right:-.625rem;
  margin-left:-.625rem;
}
@media print, screen and (min-width: 40em) {
  .grid-padding-x .grid-padding-x {
    margin-right:-.9375rem;
    margin-left:-.9375rem;
  }
}

.grid-padding-x>.cell {
  padding-right:.625rem;
  padding-left:.625rem;
}
@media print, screen and (min-width: 40em) {
  .grid-padding-x>.cell {
    padding-right:.9375rem;
    padding-left:.9375rem;
  }
}

@media print, screen and (min-width: 64em) {
  .large-up-3>.cell {
    width:33.33333%;
  }
  .large-up-4>.cell {
    width:25%;
  }
}

@media print, screen and (min-width: 64em) {
  .grid-margin-x.large-up-3>.cell {
    width:calc(33.33333% - 1.875rem);
  }
  .grid-margin-x.large-up-4>.cell {
    width:calc(25% - 1.875rem);
  }
}

.grid-padding-y {
  margin-top:-.625rem;
  margin-bottom:-.625rem;
}
@media print, screen and (min-width: 40em){
  .grid-padding-y {
    margin-top:-.9375rem;
    margin-bottom:-.9375rem;
  }
}

/* end xy grid */
div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,form,p,blockquote,th,td {
  margin:0;
  padding:0;
}
p {
  margin-bottom:1rem;
  font-size:inherit;
  line-height:1.6;
  text-rendering:optimizeLegibility;
}

/* Privacy Badger header fonts */
h1,
h2,
h3{
  font-family:"NotoSans",serif;
  font-style:normal;
  font-weight:normal;
  color:inherit;
  text-rendering:optimizeLegibility;
}

h1 {
  font-size:1.875rem;
  line-height:1.4;
  margin-top:0;
  margin-bottom:.5rem;
}
h2 {
  font-size:1.25rem;
  line-height:1.4;
  margin-top:0;
  margin-bottom:.5rem;
}
h3 {
  font-size:1.5rem;
  line-height:1.4;
  margin-top:0;
  margin-bottom:.5rem;
}

@media print, screen and (min-width: 40em) {
  h1 {
    font-size:3.5rem;
  }
  h2 {
    font-size:2.5rem;
  }
  h3 {
    font-size:1.9375rem;
  }
}

p {
  font-family:"OpenSans-Light",sans-serif;
}

a {
  line-height:inherit;
  color:#F06A0A;
  font-weight: bold;
  text-decoration:none;
  cursor:pointer;
}
a:hover,a:focus {
  color:#000;
}
a img {
  border:0;
}

hr {
  border-top:0;
  border-right:0;
  border-bottom:1px solid #cacaca;
  border-left:0;
  box-sizing:content-box;
  clear:both;
  height:0;
  margin:1.25rem auto;
  max-width:75rem;
  overflow:visible;
}
ul,ol,dl {
  margin-bottom:1rem;
  list-style-position:outside;
  line-height:1.6;
}
li {
  font-size:inherit;
}
ul {
  margin-left:1.25rem;
  list-style-type:disc;
}
ol {
  margin-left:1.25rem;
}
ul ul,ol ul,ul ol,ol ol {
  margin-left:1.25rem;
  margin-bottom:0;
}
.text-left {
  text-align:left;
}
.text-right {
  text-align:right;
}
.text-center {
  text-align:center;
}
.text-justify {
  text-align:justify;
}

/* buttons */
.button {
  display:inline-block;
  vertical-align:middle;
  margin:0 0 1rem 0;
  font-family:"OpenSans-Bold";
  padding:0.85em 1em;
  -webkit-appearance:none;
  border:1px solid transparent;
  border-radius:0;
  transition:background-color 0.25s ease-out,color 0.25s ease-out;
  font-size:0.9rem;
  line-height:1;
  text-align:center;
  cursor:pointer;
  background-color:#F06A0A;
  color:#fefefe;
}
.button:hover, .button:focus {
  background-color: #cc5a09;
  color: #000;
}

.button.large {
  font-size:1rem;
  padding:0.8rem 2rem;
  margin-top:1rem;
  text-transform:uppercase;
}

.top-bar {
  align-items: center;
  background-color: #333;
  color: white;
  display: flex;
  flex-flow: row wrap;
  flex-wrap: wrap;
  font-family: "OpenSans-Light",sans-serif;
  font-size: 0.6rem;
  justify-content: center;
  padding: 0.5rem 0.5rem 0.5rem;
}
#eff-logo {
    position: absolute;
    left: 1rem;
}
.top-bar a {
    color: #ddd;
    text-decoration: underline;
}
.top-bar a:hover {
    color: #f06a0a;
}
.top-bar img {
    max-width: 3rem;
    margin-inline-end: 0.2rem;
}
.top-bar span {
  margin: 0 4rem;
  text-align: center;
}
@media print, screen and (min-width: 40em) {
    .top-bar img {
        margin-inline-end: 1rem;
        max-width: 4rem;
    }
    .top-bar {
      flex-wrap:nowrap;
      font-size: 0.9rem;
      padding: 0.75rem 0.5rem 0.75rem;
    }
    .top-bar span {
      margin: 0 6rem;
    }
}
.top-bar-title {
  flex:0 0 auto;
  margin:0.5rem 1rem 0.5rem 0;
}

b {
    font-family: "OpenSans-Bold", sans-serif;
}

/* visibility classes */
.hide {
  display:none !important;
}
.invisible {
  visibility:hidden;
}
@media screen and (max-width: 39.9375em) {
  .hide-for-small-only {
    display:none !important;
  }
}
@media screen and (max-width: 0em), screen and (min-width: 40em) {
  .show-for-small-only {
    display:none !important;
  }
}
@media print, screen and (min-width: 40em) {
  .hide-for-medium {
    display:none !important;
  }
}
@media screen and (max-width: 39.9375em) {
  .show-for-medium {
    display:none !important;
  }
}
@media screen and (min-width: 40em) and (max-width: 63.9375em) {
  .hide-for-medium-only {
    display:none !important;
  }
}
@media screen and (max-width: 39.9375em), screen and (min-width: 64em) {
  .show-for-medium-only {
    display:none !important;
  }
}
@media print, screen and (min-width: 64em) {
  .hide-for-large {
    display:none !important;
  }
}
@media screen and (max-width: 63.9375em) {
  .show-for-large {
    display:none !important;
  }
}
@media screen and (min-width: 64em) and (max-width: 74.9375em) {
  .hide-for-large-only {
    display:none !important;
  }
}
@media screen and (max-width: 63.9375em), screen and (min-width: 75em) {
  .show-for-large-only {
    display:none !important;
  }
}

.align-right {
  justify-content:flex-end;
}
.align-center {
  justify-content:center;
}
.align-justify {
  justify-content:space-between;
}
.align-spaced {
  justify-content:space-around;
}

/* share links */
#share-container {
  margin-top: 2rem;
}
.share-links {
  display:flex;
  flex-wrap:wrap;
  gap:0.65rem;
  justify-content:center;
  margin-top: 1rem;
  z-index:3;
}
.share-links>a {
  position:relative;
  width:2.6rem;
  height:2.6rem;
  overflow:hidden;
}

.share-links>a:hover, .share-links>a:focus {
  filter: brightness(0.85);
}
.share-links>a i {
  position:relative;
  display:inline-block;
  height:2.6rem;
  width:2.6rem;
  z-index:2;
}
.share-links>a span {
  position:absolute;
  visibility:hidden;
  display:inline-block;
  top:0;
  right:0;
  z-index:1;
  opacity:0.9;
  width:12.5rem;
  height:2.6rem;
  box-sizing:content-box;
  line-height:2.6rem;
  padding-left:.45614rem;
  text-align:left;
  text-transform:uppercase;
  font-size:0.9rem;
  background-color:white;
  border-left:0.312rem solid #fff;
}
.share-links>a:first-child {
  margin-top:0;
  margin-left:0;
}
.share-links>a.share-facebook i {
  background:url("../images/facebook.svg");
}
.share-links>a.share-mastodon i {
  background:url("../images/mastodon.svg");
}
.share-links>a.share-bluesky i {
  background:url("../images/bluesky.svg");
}

/* fonts */
@font-face {
  font-family:'NotoSans';
  src:url("../fonts/NotoSans-Black.ttf") format("truetype");
  font-weight:900;
  font-style:normal;
}
@font-face {
  font-family:'OpenSans-Light';
  src:url("../fonts/OpenSans-Light.ttf") format("truetype");
  font-weight:300;
  font-style:normal;
}
@font-face {
  font-family:'OpenSans-Bold';
  src:url("../fonts/OpenSans-Bold.ttf") format("truetype");
  font-weight:600;
  font-style:normal;
}

/* pb specific */
body {
  font-size:16px;
  padding-bottom:3rem;
}

.text {
  margin:1rem 0;
}
.caret-down {
  display:flex;
  flex-flow:row wrap;
  padding:1.5rem 0 3rem;
}
.caret-down img {
  width:6rem;
  filter: brightness(0);
}
.grid-container.full {
  margin-bottom: 5rem;
}

/* Pin nudge present only on Chromium browsers */
#pb-features-pin {
  display: none;
  padding-bottom: 1rem;
}
#pb-features-pin h3 {
  font-size: 2.5rem;
}
.unpinned-chrome #pb-features-pin {
  display: flex;
}
.unpinned-chrome #pb-features-0 {
  background-color: #DFDFE6;
  flex-flow: row-reverse wrap;
  margin: 3.5rem 0 5.5rem;
  padding: 1.5rem 0 1.5rem;
}
.unpinned-chrome #pb-features-0 h3 {
  display: none;
}

/* Reduced motion version of pinning instructions */
#pinning-instructions-reduced-motion {
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;
  justify-content: center;
  align-items: center;
  width: unset;
  margin-right: unset;
  margin-left: unset;
}
#pinning-instructions-reduced-motion img {
  width: 50%;
  min-width: 200px;
  height: fit-content;
  flex-grow: 1;
}

/* Pinning instructions animation toggle */
#pb-features-pin button {
  font-family: "OpenSans-Bold", sans-serif;
  padding: 0.5rem;
  text-transform: uppercase;
  cursor: pointer;
  background-color: #DFDFE6;
  color: #333;
  border: none;
  border-bottom: 2px solid #333;
  margin-top: 0.5rem;
  display: block;
  margin-right: auto;
  margin-left: auto;
}
#pb-features-pin button:hover {
  color: #000;
  background-color: #c9c9cf;
}
#toggle-animation-icon {
  margin-right: 5px;
}

#pb-settings #toolbar-img-container {
  position: relative;
  margin-top: 0;
}
#toolbar-img-container::before {
  content: '';
  position: absolute;
  top: 20%;
  left: 50%;
  transform: translateX(-50%);
  width: 100vw;
  height: 60%;
  background-color: #DFDFE6;
  z-index: -1;
}

#badger-img-container {
  position: relative;
}
#badger-img-container::before {
  content: '';
  position: absolute;
  top: 20%;
  left: 50%;
  transform: translateX(-50%);
  width: 100vw;
  height: 60%;
  background-color: #EC9329;
  z-index: -1;
}

#pb-settings div {
  display: flex;
  flex-direction: column;
  align-items: center;
  font-size: 17px;
  padding: 1rem;
  margin: 2rem 0 3rem;
  width: 100%;
}

#toolbar-icon {
  max-width: 550px;
  width: 90%;
}
[id*="pb-features-"] img {
  max-width: 400px;
  width: 100%;
}
[id*="pb-features-"] {
  display:flex;
  flex-flow:row wrap;
  padding:3rem 0 2rem;
}
#pb-features-0 {
    gap: 1.5rem;
    padding-bottom: 4rem;
}
@media print, screen and (min-width: 40em) {
  [id*="pb-features-"] {
    gap: 1.5rem;
  }
  #pb-features-0 {
    padding:5rem 0;
  }
}
[id*="pb-features-"] div {
  width:calc(100% - 1.875rem);
  margin-right:.9375rem;
  margin-left:.9375rem;
  text-align:center;
  align-content: center;
}
@media print, screen and (min-width: 40em) {
  [id*="pb-features-"] div {
    width:calc(40% - 1.875rem);
    margin-right:.9375rem;
    margin-left:.9375rem;
  }
}
@media print, screen and (min-width: 50em) {
  [id*="pb-features-"] div {
    width:calc(33.33333% - 1.875rem);
    margin-right:.9375rem;
    margin-left:.9375rem;
  }
}
[id*="pb-features-"] h3 {
  font-size:1.5rem;
  text-align:center;
  margin-top:1rem;
}
@media print, screen and (min-width: 40em) {
    [id*="pb-features-"] h3 {
        text-align: start;
        font-size: 1.9375rem;
    }
}
[id*="pb-features-"] .text, [id*="pb-features-"] p {
    text-align: start;
}

#pb-donate, #pb-donate-button, #pb-donate-help {
  display:flex;
  flex-flow:row wrap;
}
#pb-donate img {
  max-width:200px;
}
@media print, screen and (min-width: 40em) {
  #pb-donate img {
    max-width:250px;
  }
}
#pb-donate div:not(#badger-img-container), #pb-donate-button div, #pb-donate-help span {
  width:calc(100% - 1.875rem);
  margin-right:.9375rem;
  margin-left:.9375rem;
}
@media print, screen and (min-width: 40em) {
  #pb-donate div:not(#badger-img-container), #pb-donate-button div, #pb-donate-help span {
    width:calc(33.33333% - 1.875rem);
    margin-right:.9375rem;
    margin-left:.9375rem;
  }

}
#pb-donate h3 {
  font-size:1.5rem;
  text-align:center;
  margin-top:1rem;
}
@media print, screen and (min-width: 40em) {
  #pb-donate h3 {
    font-size:1.9375rem;
  }
}
#pb-donate-help span {
  text-align:center;
}
#pb-donate {
  padding-top: 4rem;
}

#pb-privacy-policy, #pb-privacy-policy a {
  color:#707070;
  font-family:"OpenSans-Light",sans-serif;
  font-size:0.9rem;
  text-align:center;
}

#pb-donate-help {
  margin-top: 2rem;
  margin-bottom:0;
}

#pb-donate-help h3 {
  margin-bottom: 0;
}

@media (prefers-color-scheme: dark) {
  :root {
    color-scheme: dark;
  }

  body, #pb-privacy-policy {
    background-color: #222;
    color: #ddd;
  }

  a:hover, a:focus {
    color: #ddd;
  }

  .button:hover, .button:focus {
    color: #000;
  }

  .unpinned-chrome #pb-features-0, #toolbar-img-container::before {
    background-color: #333;
  }

  .share-links > a span {
    background-color: #555;
    border-left: 0.4275rem solid #555;
    color: #fff;
  }

  .caret-down img {
    /* Filter generated by https://angel-rs.github.io/css-color-filter-generator/ to convert color to #ddd */
    filter: brightness(0) saturate(100%) invert(98%) sepia(0%) saturate(5139%) hue-rotate(58deg) brightness(126%) contrast(73%);
  }
}
